import requests
from .hash import hasher
from .vars import HEADERS
import os

URL_BASE = 'https://api.papermc.io'


def download():
    print("Querying version list from PaperMC...")

    versions = requests.get(URL_BASE + '/v2/projects/paper', headers=HEADERS, allow_redirects=True)

    versions = versions.json()
    ver_list = versions.get("versions")
    print("Current Version list for PaperMC:")
    print(ver_list)

    selected_version = input("Choose a version: ")

    builds = requests.get(URL_BASE + '/v2/projects/paper/versions/' + selected_version, headers=HEADERS,
                          allow_redirects=True)
    builds = builds.json()
    build_list = builds.get("builds")
    build = build_list[-1]
    print("Latest Build: " + str(build))

    build_info = requests.get(URL_BASE + "/v2/projects/paper/versions/" + selected_version + "/builds/" + str(build),
                              headers=HEADERS, allow_redirects=True)
    build_info = build_info.json()
    build_info = build_info.get("downloads")
    build_info = build_info.get("application")
    filename = build_info["name"]

    print("File to download: " + filename)

    sha256 = build_info["sha256"]

    print("Downloading...")

    response = requests.get(
        URL_BASE + "/v2/projects/paper/versions/" + selected_version + "/builds/" + str(
            build) + "/downloads/" + filename, allow_redirects=True)

    with open(filename, 'wb') as f:
        f.write(response.content)

    # Hash it to be sure nothing funky happened on the way.

    new_hash = hasher(filename, 256)
    print("Hash of the downloaded file: " + new_hash)  # Get the hexadecimal digest of the hash

    if new_hash == str(sha256):
        print("Hash OK")
    else:
        print("Bad hash, incorrect or unexpected file. Exiting.")
        exit()

    os.system("chmod a+x " + filename)

    return filename, selected_version



import json
from .providers import download
from .vars import HEADERS
import os


def update_plugins(init):
    print("Grabbing server info...")

    with open("server.json", "r") as read_file:
        server_info = json.load(read_file)

    plugins = server_info["plugins"]
    if not os.path.exists("./plugins/update"):
        os.makedirs("./plugins/update")
    if plugins:
        for x in plugins:
            download(x, HEADERS, init)
    else:
        print(
            "No plugins are available to be updated. Either you have no installed plugins or the plugins haven't been properly added to server.json.")



import argparse
import json
import os


from .paper import download
from .flags import make_script
from .updatePlugins import update_plugins
from .install_plugin import install_plugin
from .vars import BASE_DIR, CURRENT_DIR


# import updatePlugins
def create():
    print("Creating the server...")
    info = download()
    print(info)
    filename = info[0]
    version = info[1]
    memory_limit = int(input("Enter the amount of memory the server should have, in GB: "))
    make_script(memory_limit, filename)
    with open(BASE_DIR + "/static/base.json", "r") as read_file:
        data = json.load(read_file)
    data["memory"] = memory_limit
    data["version"] = version
    with open("server.json", "w") as write_file:
        json.dump(data, write_file, indent=4)
    with open("eula.txt", "w") as eula_file:
        eula_file.write("eula=true")


def update():
    print("Updating server file...")
    with open("server.json", "r") as read_file:
        server_info = json.load(read_file)
    print("Current version: " + server_info["version"])
    info = download()
    filename = info[0]
    version = info[1]
    memory_limit = server_info["memory"]
    make_script(memory_limit, filename)
    server_info["version"] = version
    with open("server.json", "w") as write_file:
        json.dump(server_info, write_file, indent=4)


def plugin_update():
    print("Updating server plugins...")
    update_plugins(False)


def install():
    name = input("Enter the plugin's name: ")
    provider = input("Enter the plugin's provider: (github, modrinth, hangar, spigot, or geyser): ")
    identifier = input("Enter the plugin's id, slug, or github repo: ")
    if not os.path.exists("./plugins/update"):
        os.makedirs("./plugins/update")
    install_plugin(name, provider, identifier)


def update_full():
    update()
    plugin_update()


def initialize():
    update_plugins(True)


parser = argparse.ArgumentParser()
subparsers = parser.add_subparsers(title="subcommands", help='help with subcommands', required=True)

parser_create = subparsers.add_parser("create",
                                      help="download a server for the first time and make a config and a startup file")
parser_create.set_defaults(func=create)

parser_update = subparsers.add_parser("update", help="update the papermc jar file and the startup script")
parser_update.set_defaults(func=update)

parser_plugin_update = subparsers.add_parser("update-plugins",
                                             help="update all currently installed plugins from their IDs in the config file")
parser_plugin_update.set_defaults(func=plugin_update)

parser_install = subparsers.add_parser("install", help="download the specified plugin and install it in the server")
parser_install.set_defaults(func=install)

parser_update_full = subparsers.add_parser("update-full",
                                           help="run update and then update-plugins in one command")
parser_update_full.set_defaults(func=update_full)

parser_update_full = subparsers.add_parser("initialize",
                                           help="initialize all plugins from the config file (to facilitate easy server transplanting)")
parser_update_full.set_defaults(func=initialize)


def main():
    print("Running in: " + CURRENT_DIR)
    args = parser.parse_args()
    args.func()



import os

BASE_DIR = os.path.dirname(__file__)
HEADERS = {
    'Content-Type': 'application/json',
    'User-Agent': 'transdryad/PaperMC-Updater/1.0 (viswanath.hazel@gmail.com)',
    'accept': 'application/json',
}
CURRENT_DIR = os.getcwd()



import requests
from .hash import hasher

import os


def plugin_version_exists(jarfile):
    return os.path.exists("./plugins/" + jarfile)


def download(plugin, headers, install):
    name = plugin["name"]
    provider = plugin["provider"]
    plugin_id = plugin["id"]
    if install:
        path = "./plugins/"
    else:
        path = "./plugins/update/"

    if provider == "modrinth":
        print("Downloading " + name + " from Modrinth...")
        params = {
            'loaders': '["paper"]'
        }
        versions = requests.get('https://api.modrinth.com/v2/project/' + plugin_id + "/version", params=params,
                                headers=headers, allow_redirects=True)
        versions = versions.json()
        expected_hash = versions[0]["files"][0]["hashes"]["sha512"]
        url = versions[0]["files"][0]["url"]
        filename = versions[0]["files"][0]["filename"]
        if plugin_version_exists(filename):
            print("No updates")
            return
        filename = path + filename
        response = requests.get(url)
        with open(filename, 'wb') as f:
            f.write(response.content)

        new_hash = hasher(filename, 512)

        if not new_hash == str(expected_hash):
            print("Bad hash, incorrect or unexpected file. Please Manually check the file.")

    elif provider == "github":
        print("Downloading " + name + " from Github...")
        response = requests.get("https://api.github.com/repos/" + plugin_id + "/releases", headers=headers,
                                allow_redirects=True)
        releases = response.json()
        release = releases[0]  # Is a dictionary
        newest_version = release["name"]
        print(newest_version)
        filename = newest_version + "-" + release["assets"][0]["name"]
        if plugin_version_exists(filename):
            print("No updates")
            return
        filename = path + filename
        download_url = release["assets"][0]["browser_download_url"]

        new_file = requests.get(download_url)
        with open(filename, 'wb') as f:
            f.write(new_file.content)
    elif provider == "spigot":
        print("Downloading " + name + " from Spigot...")
        date = requests.get("https://api.spiget.org/v2/resources/" + plugin_id, headers=headers,
                            allow_redirects=True).json()["updateDate"]
        filename = name + "-" + str(date) + ".jar"
        if plugin_version_exists(filename):
            print("No updates")
            return
        filename = path + filename
        response = requests.get("https://api.spiget.org/v2/resources/" + plugin_id + "/download", headers=headers,
                                allow_redirects=True)
        with open(filename, 'wb') as f:
            f.write(response.content)
    elif provider == "geyser":
        print("Downloading " + name + " from the GeyserMC website...")
        version = requests.get("https://download.geysermc.org/v2/projects/" + plugin_id, headers=headers, allow_redirects=True).json()["versions"][-1]
        filename = name + "-" + version + ".jar"
        if plugin_version_exists(filename):
            print("No updates")
            return
        filename = path + filename
        response = requests.get(
            "https://download.geysermc.org/v2/projects/" + plugin_id + "/versions/latest/builds/latest/downloads/spigot",
            headers=headers, allow_redirects=True)
        with open(filename, 'wb') as f:
            f.write(response.content)
    elif provider == "hangar":
        print("Downloading " + name + " from Hangar...")
        response = requests.get('https://hangar.papermc.io/api/v1/projects/' + plugin_id + "/versions", headers=headers,
                                allow_redirects=True)
        response = response.json()

        url = response["result"][0]["downloads"]["PAPER"]["downloadUrl"]
        filename = response["result"][0]["downloads"]["PAPER"]["fileInfo"]["name"]
        if plugin_version_exists(filename):
            print("No updates")
            return
        filename = path + filename
        response = requests.get(url)
        with open(filename, 'wb') as f:
            f.write(response.content)



import hashlib


def hasher(filename, size):
    global file_hash
    file: str = filename  # Location of the file
    BLOCK_SIZE = 131072  # The size of each read from the file

    if size == 256:
        file_hash = hashlib.sha256()
    elif size == 512:
        file_hash = hashlib.sha512()

    with open(file, 'rb') as f:  # Open the file to read its bytes
        fb = f.read(BLOCK_SIZE)  # Read from the file. Take in the amount declared above
        while len(fb) > 0:  # While there is still data being read from the file
            file_hash.update(fb)  # Update the hash
            fb = f.read(BLOCK_SIZE)  # Read the next block from the file

    return str(file_hash.hexdigest())



from .providers import download
from .vars import HEADERS
import json


def install_plugin(name, provider, identifier):
    with open("server.json", "r") as read_file:
        server_info = json.load(read_file)
    plugin = {"name": name, "provider": provider, "id": identifier}
    if plugin in server_info["plugins"]:
        print("You may not install multiple copies of the same plugin!")
        return
    server_info["plugins"].append(plugin)
    with open("server.json", "w") as write_file:
        json.dump(server_info, write_file, indent=4)
    download(plugin, HEADERS, True)



import os


def gen_script(gb, jar_name):
    beginning = "#!/bin/bash \njava "
    aikars = " --add-modules=jdk.incubator.vector -XX:+UseG1GC -XX:+ParallelRefProcEnabled -XX:MaxGCPauseMillis=200 -XX:+UnlockExperimentalVMOptions -XX:+DisableExplicitGC -XX:+AlwaysPreTouch -XX:G1HeapWastePercent=5 -XX:G1MixedGCCountTarget=4 -XX:InitiatingHeapOccupancyPercent=15 -XX:G1MixedGCLiveThresholdPercent=90 -XX:G1RSetUpdatingPauseTimePercent=5 -XX:SurvivorRatio=32 -XX:+PerfDisableSharedMem -XX:MaxTenuringThreshold=1 -Dusing.aikars.flags=https://mcflags.emc.gs -Daikars.new.flags=true -XX:G1NewSizePercent=30 -XX:G1MaxNewSizePercent=40 -XX:G1HeapRegionSize=8M -XX:G1ReservePercent=20 "
    mb = int(gb) * 1024
    memory = "-Xms" + str(mb) + "M -Xmx" + str(mb) + "M"
    end = "-jar " + jar_name + " --nogui"
    sh = beginning + memory + aikars + end
    return sh


def write_script(script):
    file = open(r"start.sh", "w")
    file.write(script)
    file.close()


def make_script(gb, jar_name):
    write_script(gen_script(gb, jar_name))
    os.system("chmod a+x " + "start.sh")
